// IntMars -- Mars central meridian using integer arithmatic,
// since the Palm doesn't do floating point well.
// Copyright 2001 by Akkana Peck.

#include <PalmOS.h>

#include "pilrc.h"
#include "timezone.c"

int GetCentralMeridian(long seconds);

// Palm GNU SDK can't handle multiple object files, sigh.
#include "features.h"
#include "intmars.c"

#define XSTART 30
#define YSTART 45
#define XSIZE 100
#define YSIZE 115
#define CHARWIDTH 5
#define WORDHEIGHT 11
#define LIMB_CUTOFF 75

void
ShowFeature(feature_t* feature, int meridian, int EW, int NS)
{
    int y = (90 - feature->latitude) * YSIZE / 180;
    int x = meridian - feature->longitude - 90;
    while (x < 0) x += 180;
    while (x > 180) x -= 180;
    x = x * XSIZE / 180;
    if (EW) x = XSIZE - x;
    if (NS) y = YSIZE - y;
    y += YSTART - WORDHEIGHT;
    x += XSTART - CHARWIDTH;
    //printf("%s (%d, %d)\n", feature->shortname, x, y);
    if (feature->lightP)
        WinDrawChars( feature->shortname, StrLen(feature->shortname), x, y);
    else
        WinDrawInvertedChars( feature->shortname, StrLen(feature->shortname),
                              x, y );
}

void
DisplayFeaturesVisible(int timeOffset, int timezone, int EW, int NS)
{
    int NumFeatures = sizeof (Features) / sizeof (*Features);
    int startLong;
    int endLong;
    int i;
    RectangleType rect;
    char Buffer[200];
    long secOffset;
    int meridian;
    DateTimeType datetime;
    UInt32 nowseconds = TimGetSeconds();

    // Need to convert that to be relative to May 13 0 UT.
    // 3074528425 when unix says 991708605.
    // Unix has to subtract 992476800,
    // so Palm should subtract 992476800 - 991708605 + 3074528425.
    secOffset = nowseconds - (long)3075321820LU - (long)timezone*3600
                + (long)timeOffset * 3600;

    meridian = GetCentralMeridian(secOffset);

    startLong = (meridian + 360 - LIMB_CUTOFF) % 360;
    endLong = (meridian + LIMB_CUTOFF) % 360;

    rect.topLeft.x = XSTART;
    rect.topLeft.y = YSTART;
    rect.extent.x = XSIZE;
    rect.extent.y = XSIZE;

    WinEraseWindow();
    WinDrawChars("======= Mars 2001 =======", 25, 15, 0);

    TimSecondsToDateTime(nowseconds+ timeOffset * 3600, &datetime);
    StrPrintF(Buffer, "Now%s%d hrs, %d:%d %d/%d/%d TZ %d",
              timeOffset >= 0 ? "+" : "", timeOffset,
              datetime.hour, datetime.minute,
              datetime.month, datetime.day, datetime.year, timezone);
    WinDrawChars(Buffer, StrLen(Buffer), 8, 15);
    //StrPrintF(Buffer, "TZ: %d", timezone);
    //WinDrawChars(Buffer, StrLen(Buffer), 95, 30);

    StrPrintF(Buffer, "Central Meridian: %d%s", meridian,
              (EW && NS) ? " (rotated)" : ((EW || NS) ? " (flipped)" : "") );
    i = StrLen(Buffer);
    WinDrawChars(Buffer, i, 90 - i * CHARWIDTH / 2, 25);

    //WinDrawRectangle(&rect, 5);
    WinDrawRectangleFrame(boldRoundFrame, &rect);
    // WinDrawChar doesn't work right on the Visor!
    WinDrawChars(NS ? "S" : "N",
                 1, XSTART + XSIZE / 2, YSTART-WORDHEIGHT/2-1);
    WinDrawChars(NS ? "N" : "S",
                 1, XSTART + XSIZE / 2, YSTART+XSIZE-WORDHEIGHT/2);
    WinDrawChars(EW ? "E" : "W",
                 1, XSTART - CHARWIDTH/2, YSTART-WORDHEIGHT/2-1 + XSIZE/2);
    WinDrawChars(EW ? "W" : "E",
                 1, XSTART + XSIZE, YSTART-WORDHEIGHT/2-1 + XSIZE/2);

    //printf("%d - %d\n", startLong, endLong);

    for (i=0; i < NumFeatures; ++i)
    {
        if (( startLong < endLong
              && Features[i].longitude >= startLong
              && Features[i].longitude <= endLong) ||
            (startLong > endLong
             && (Features[i].longitude >= startLong
                 || Features[i].longitude <= endLong)) )
            ShowFeature(&(Features[i]), meridian, EW, NS);
    }
}

/* -----------------------------------------------------
 * PilotMain is called by the startup code and implements 
 * a simple event handling loop.
 * -----------------------------------------------------
 */
UInt32
PilotMain (UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
    EventType event;

    if (cmd == sysAppLaunchCmdNormalLaunch)
    {
        Char buffer[200];
        int timezone;
        int penX = 0;
        int timeOffset = -1;
        int EW = 0;
        int NS = 0;

        timezone = GetTimeZoneOffset();

        FrmGotoForm(mainForm);

        // Main event loop:
        do
        {
            UInt16 error;
            FormType* pfrm;
            int dX;

            // Doze until an event arrives.
            EvtGetEvent( &event, evtWaitForever );

            // System gets first chance to handle the event.
            if (SysHandleEvent( &event )) continue;
            if (MenuHandleEvent(0, &event, &error)) continue;
            //if (ApplicationHandleEvent( &event )) continue;

            switch (event.eType)
            {
              case penDownEvent:
                //printf("pen down: (%d, %d)\n", event.screenX, event.screenY);
                penX = event.screenX;
                break;

              case penUpEvent:
                //printf("pen up: %d (%d, %d)\n", penX, event.screenX, event.screenY);
                if (penX < 0)             // at start, or after menu event
                {
                    penX = 0;
                    //printf("penX was negative, ignoring\n");
                    continue;
                }
                if (event.screenY > 160)  // happens at exit
                    continue;

                dX = event.screenX - penX;
                //printf("Pen moved %d!\n", dX);
                if (dX == 0)
                    timeOffset = 0;
                else if ((dX < 0 && !EW) || (dX > 0 && EW))
                    --timeOffset;
                else if ((dX > 0 && !EW) || (dX < 0 && EW))
                    ++timeOffset;

                DisplayFeaturesVisible(timeOffset, timezone, EW, NS);
                break;

              case menuEvent:
                  switch (event.data.menu.itemID)
                  {
                    case flipEW:
                      EW = !EW;
                      DisplayFeaturesVisible(timeOffset, timezone, EW, NS);
                      break;
                    case flipNS:
                      NS = !NS;
                      DisplayFeaturesVisible(timeOffset, timezone, EW, NS);
                      break;
                    case noFlip:
                      EW = NS = 0;
                      DisplayFeaturesVisible(timeOffset, timezone, EW, NS);
                      break;
                    case aboutMenu:
                    default:
                      FrmAlert(aboutAlert);
                      break;
                  }
                  penX = -1;
                  break;

              case frmLoadEvent:
                  FrmSetActiveForm(FrmInitForm(event.data.frmLoad.formID));
                  break;
	
              case frmOpenEvent:
                  pfrm = FrmGetActiveForm();
                  FrmDrawForm(pfrm);
                  break;

              default:
                if (FrmGetActiveForm())
                    FrmHandleEvent(FrmGetActiveForm(), &event);
            }

            // Return from PilotMain when an 
            // appStopEvent is received.
        } while (event.eType != appStopEvent);
    }

    return;
}

